# SPDX-FileCopyrightText: 2023-present pbousquets <pbousquetsm@gmail.com>
#
# SPDX-License-Identifier: MIT
import sys

if __name__ == "__main__":
    from phyfum.cli import phyfum

    sys.exit(phyfum())
